// LVSelStateDlg.cpp : implementation file
//

// Sample:  LVSelState
// Purpose: Illustrates using custom image lists vs. check box style
// in a list control
// Author:  Arjay 12-15-04

// Features
// 1) Illustrates safe string copy using strsafe.h
// 2) Uses virtual list view
// 3) Uses secondary string to populate std::vector list
// 4) Shows one method of spawning a secondary thread
// and processing messages while waiting for the thread to finish
// 5) Uses a CAutoLockT class to show how to easily provide thread
// safe locking of objects (such as std::vector) shared between
// threads
// 6) Illustrates one method of cancelling a secondary
// thread while it's performing work

// Finally, and the reasons I've created this sample.
// 7) Illustrates how to use a custom image list rather than the
// default check box style... and
// 8) How to intercept click messages to change image state
// when a user clicks on the image icon.

#include "stdafx.h"
#include "LVSelState.h"
#include "LVItemData.h"
#include "LVItemDataMgr.h"
#include "LVSelStateDlg.h"
#include ".\lvselstatedlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CLVSelStateDlg dialog
CLVSelStateDlg::CLVSelStateDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLVSelStateDlg::IDD, pParent)
    , m_sStatus( _T("") )
    , m_uMaxCount( 10000 ) // Initial Max Count (real max is 1,000,000)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CLVSelStateDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_LIST, m_ctlList);
    DDX_Control(pDX, ID_RETRIEVE, m_btnRetrieve);
    DDX_Text( pDX, IDC_STATUS, m_sStatus );
    DDX_Control(pDX, IDCANCEL, m_btnCancel);
    DDX_Text( pDX, IDC_MAXCOUNT, m_uMaxCount);
    DDX_Control(pDX, IDC_MAX_COUNT_SPIN, m_ctlMaxCountSpin);
}

BEGIN_MESSAGE_MAP(CLVSelStateDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
    ON_NOTIFY(NM_CLICK, IDC_LIST, OnNMClickList)
    ON_BN_CLICKED(ID_RETRIEVE, OnBnClickedRetrieve)
    ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
    ON_NOTIFY(LVN_GETDISPINFO, IDC_LIST, OnLvnGetdispinfoList)
    ON_WM_SETCURSOR()
END_MESSAGE_MAP()


// CLVSelStateDlg message handlers

BOOL CLVSelStateDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

    // Create and set the image list for the list control
    m_ImageList.Create(IDB_CHECK, 16, 16, RGB(255, 255, 255));
    m_ctlList.SetImageList(&m_ImageList, LVSIL_SMALL);
	m_ImageList.Detach();

    // Add some columns to the list control
    m_ctlList.InsertColumn( 0, _T("Column 1") );
    m_ctlList.InsertColumn( 1, _T("Column 2") );
    m_ctlList.InsertColumn( 2, _T("Column 3") );

    // Set the column widths
    m_ctlList.SetColumnWidth( 0, 100 );
    m_ctlList.SetColumnWidth( 1, 100 );
    m_ctlList.SetColumnWidth( 2, LVSCW_AUTOSIZE_USEHEADER );

    // Set the range on the spin control (max 1,000,000 items)
    m_ctlMaxCountSpin.SetRange32( 0, 1000000 );
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLVSelStateDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display
// while the user drags the minimized window.
HCURSOR CLVSelStateDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

// Status text helper method
void CLVSelStateDlg::SetStatusText( LPCTSTR szText )
{
    m_sStatus = szText;
    m_sStatus = _T(" ") + m_sStatus;
    UpdateData( FALSE );
}

//
// Button Handlers
//

void CLVSelStateDlg::OnBnClickedCancel()
{
    m_btnCancel.EnableWindow( FALSE );
    m_btnRetrieve.EnableWindow( FALSE );

    SetStatusText( _T("Clearing data - standby...") );

    CWaitCursor wait;

    // Clear the List control
    m_ctlList.DeleteAllItems();
    m_ctlList.UpdateWindow();
   
    wait.Restore();

    // Interrupt the secondary thread and delete the items
    m_LVItemDataMgr.Cancel();

    // Close the dialog
    OnCancel();
}

void CLVSelStateDlg::OnBnClickedRetrieve()
{
    m_btnRetrieve.EnableWindow( FALSE );
    UpdateData( TRUE );

    CWaitCursor wait;

    // Clear list ctrl items
    SetStatusText( _T("Clearing the list control...") );
    m_ctlList.DeleteAllItems();
    m_ctlList.UpdateWindow();

    SetStatusText( _T("Retrieving data...") );

    wait.Restore();
    m_LVItemDataMgr.Retrieve( m_uMaxCount );

    SetStatusText( _T("Populating list control..") );
    
    m_LVItemDataMgr.Populate( m_ctlList );
    wait.Restore();
   	m_ctlList.UpdateWindow();

    // Enable the buttons
    m_btnCancel.EnableWindow( TRUE );
    m_btnRetrieve.EnableWindow( TRUE );

	SetStatusText( _T("") );

}

//
// List Control Notify Message Handlers
//

// Method: OnNMClickList
// Purpose: Handle the Left Button click.  We handle this message
// in order to change the item check state.
void CLVSelStateDlg::OnNMClickList(NMHDR *pNMHDR, LRESULT *pResult)
{
   	NMLISTVIEW     *pnmListView = (NMLISTVIEW *)pNMHDR;

    CLVItemData*    pLVItemData = NULL;
    UINT            uFlags = 0;

    // See if we've clicked on an item.  Retrieve the flags
    // and the item if we have
    int  nItem = m_ctlList.HitTest( pnmListView->ptAction, &uFlags );

    if( INVALID_INDEX != nItem )
    {
        // We've got a valid LVItem, so retrieve the associated
        // CVLItemData object
        if( NULL != ( pLVItemData = reinterpret_cast<CLVItemData*>(m_ctlList.GetItemData( nItem )) ))
        {
            // Change the check state
            pLVItemData->SetSelectedState( uFlags, pnmListView );
            m_ctlList.Update( nItem );  // tell the control to repaint this item
        }
    }

    *pResult = 0;
}


// Method: OnLvnGetdispinfoList
// Purpose: Handle the List Control Virtual callback.  This handler gets
// called everytime the list control needs to update its [visible] display
// data.  
// 
// NOTE: To Virtual or not
// Why use a virtual list control?  
// It's faster than a regular list control because the regular list control
// makes a copy of all the string data and stores the data directly in
// the control.  This causes more memory to be used (because if you keep the
// data in a list like I am, you have the list data AND the control data).
// Search MSDN on "Virtual List-View Style" for more info.
void CLVSelStateDlg::OnLvnGetdispinfoList(NMHDR *pNMHDR, LRESULT *pResult)
{
    NMLVDISPINFO *pDispInfo = reinterpret_cast<NMLVDISPINFO*>(pNMHDR);

    // Get the list control item
    LVITEM* pItem = &(pDispInfo)->item;

    // Retrieve the CLVItemData object associated with this item
	CLVItemData* pLVItemData = reinterpret_cast<CLVItemData*>(pItem->lParam);

    if( NULL != pLVItemData )
    {
        // Let the item method fill out the display data 
        pLVItemData->GetListViewItemInfo( pItem );
    }

    *pResult = 0;
}